@@ -1,4 +1,4 @@
-from typing import Generator, List, Any
+from typing import Generator, List, Any, Dict, IO
 
 import pandas as pd
 import ijson
@@ -7,19 +7,23 @@ from .dataset import Dataset
 from cleanlab_studio.cli.types import RecordType
 
 
-class JsonDataset(Dataset):
+class JsonDataset(Dataset[IO[bytes]]):
+    READ_ARGS: Dict[str, str] = {"mode": "rb"}
+
     def read_streaming_records(self) -> Generator[RecordType, None, None]:
-        with open(self.filepath, "rb") as f:
+        with self.fileobj() as f:
             for r in ijson.items(f, "item"):
                 yield r
 
     def read_streaming_values(self) -> Generator[List[Any], None, None]:
-        with open(self.filepath, "rb") as f:
+        with self.fileobj() as f:
             for r in ijson.items(f, "item"):
                 yield r.values()
 
     def read_file_as_dataframe(self) -> pd.DataFrame:
-        df = pd.read_json(self.filepath, convert_axes=False, convert_dates=False).T
+        with self.fileobj() as f:
+            df = pd.read_json(f, orient="records", convert_axes=False, convert_dates=False)
+
         df.index = df.index.astype("str")
         df["id"] = df.index
         return df
