@@ -37,6 +37,7 @@ from cleanlab_studio.cli.dataset.image_utils import (
     image_file_exists,
     get_image_filepath,
 )
+from cleanlab_studio.cli.dataset.schema_helpers import get_dataset_filepath_columns
 from cleanlab_studio.cli.dataset.schema_types import (
     PYTHON_TYPES_TO_READABLE_STRING,
     DATA_TYPES_TO_PYTHON_TYPES,
@@ -51,13 +52,15 @@ from cleanlab_studio.cli.dataset.upload_types import (
     warning_to_readable_name,
 )
 from cleanlab_studio.cli.types import (
-    RecordType,
+    DatasetFileExtension,
     Modality,
+    RecordType,
 )
 from cleanlab_studio.cli.util import (
     is_null_value,
     dump_json,
     init_dataset_from_filepath,
+    init_dataset_from_fileobj,
     get_file_size,
 )
 
@@ -81,8 +84,6 @@ def validate_and_process_record(
     schema: Schema,
     seen_ids: Set[str],
     existing_ids: Set[str],
-    base_directory: pathlib.Path = pathlib.Path(""),
-    disable_filepath_checks: bool = False,
 ) -> Tuple[Optional[RecordType], Optional[str], Optional[RowWarningsType]]:
     """
     Validate the row against the provided schema; generate warnings where issues are found
@@ -152,26 +153,6 @@ def validate_and_process_record(
                         " parsable by pandas.Timestamp()",
                         ValidationWarning.TYPE_MISMATCH,
                     )
-            elif col_feature_type == FeatureType.filepath:
-                if schema.metadata.modality == Modality.image and not disable_filepath_checks:
-                    image_filepath = get_image_filepath(base_directory, column_value)
-                    if not image_file_exists(image_filepath):
-                        msg, warn_type = (
-                            f"{column_name}: unable to find file at specified filepath {image_filepath}. "
-                            f"Filepath must be absolute or relative to the directory containing your dataset file.",
-                            ValidationWarning.MISSING_FILE,
-                        )
-                        warnings[warn_type].append(msg)
-                        return None, row_id, warnings
-                    else:
-                        if not image_file_readable(image_filepath):
-                            msg, warn_type = (
-                                f"{column_name}: could not open file at {image_filepath}.",
-                                ValidationWarning.UNREADABLE_FILE,
-                            )
-                            warnings[warn_type].append(msg)
-                            return None, row_id, warnings
-
             else:
                 if col_type == DataType.string:
                     row[column_name] = str(column_value)  # type coercion
@@ -271,7 +252,7 @@ def echo_log_warnings(log: WarningLog) -> None:
 
 
 def validate_rows(
-    dataset_filepath: str,
+    dataset: Union[Dataset[IO[bytes]], Dataset[IO[str]]],
     schema: Schema,
     log: WarningLog,
     upload_queue: "queue.Queue[Optional[List[Any]]]",
@@ -289,8 +270,6 @@ def validate_rows(
     existing_ids = set() if existing_ids is None else set([str(x) for x in existing_ids])
     seen_ids: Set[str] = set()
 
-    dataset = init_dataset_from_filepath(dataset_filepath)
-
     process_dataset(
         dataset=dataset,
         schema=schema,
@@ -298,7 +277,6 @@ def validate_rows(
         existing_ids=existing_ids,
         log=log,
         upload_queue=upload_queue,
-        dataset_dir=pathlib.Path(dataset_filepath).parent,
     )
     upload_queue.put(None, block=True)
 
@@ -310,6 +288,7 @@ async def upload_rows(
     schema: Schema,
     upload_queue: "queue.Queue[Optional[List[Any]]]",
     rows_per_payload: int,
+    filepath_columns: List[str],
 ) -> None:
     """Gets rows from upload queue and uploads to API.
 
@@ -340,6 +319,7 @@ async def upload_rows(
                             dataset_filepath=dataset_filepath,
                             schema=schema,
                             rows=payload,
+                            filepath_columns=filepath_columns,
                         )
                     )
                 )
@@ -365,19 +345,24 @@ async def upload_rows(
                     dataset_filepath=dataset_filepath,
                     schema=schema,
                     rows=payload,
+                    filepath_columns=filepath_columns,
                 )
             )
 
         await asyncio.gather(*upload_tasks)
 
 
-def check_filepath_column(modality: Modality, dataset_filepath: str, filepath_column: str) -> None:
+def check_filepath_column(
+    dataset: Union[Dataset[IO[bytes]], Dataset[IO[str]]],
+    modality: Modality,
+    dataset_filepath: str,
+    filepath_column: str,
+) -> None:
     """
     Check the filepath column of a dataset to see if any of the filepaths are invalid.
     If >0 filepaths are invalid, print the number of invalid filepaths and prompt user for confirmation about
     outputting filepaths to console.
     """
-    dataset = init_dataset_from_filepath(dataset_filepath)
     dataset_dir = pathlib.Path(dataset_filepath).parent
 
     if filepath_column not in dataset.get_columns():
@@ -431,7 +416,29 @@ def check_filepath_column(modality: Modality, dataset_filepath: str, filepath_co
             abort("Dataset upload aborted.")
 
 
-def upload_dataset(
+def upload_dataset_from_stream(
+    api_key: str,
+    dataset_id: str,
+    fileobj: Union[IO[str], IO[bytes]],
+    schema: Schema,
+    dataset_file_extension: DatasetFileExtension,
+    existing_ids: Optional[Collection[str]] = None,
+    output: Optional[str] = None,
+    payload_size: float = 10,
+) -> None:
+    dataset = init_dataset_from_fileobj(fileobj, dataset_file_extension)
+    upload_dataset(
+        api_key=api_key,
+        dataset_id=dataset_id,
+        dataset=dataset,
+        schema=schema,
+        existing_ids=existing_ids,
+        output=output,
+        payload_size=payload_size,
+    )
+
+
+def upload_dataset_from_filename(
     api_key: str,
     dataset_id: str,
     filepath: str,
@@ -439,6 +446,29 @@ def upload_dataset(
     existing_ids: Optional[Collection[str]] = None,
     output: Optional[str] = None,
     payload_size: float = 10,
+) -> None:
+    dataset = init_dataset_from_filepath(filepath)
+    upload_dataset(
+        api_key=api_key,
+        dataset_id=dataset_id,
+        dataset=dataset,
+        schema=schema,
+        existing_ids=existing_ids,
+        output=output,
+        payload_size=payload_size,
+        filepath=filepath,
+    )
+
+
+def upload_dataset(
+    api_key: str,
+    dataset_id: str,
+    dataset: Union[Dataset[IO[bytes]], Dataset[IO[str]]],
+    schema: Schema,
+    existing_ids: Optional[Collection[str]] = None,
+    output: Optional[str] = None,
+    payload_size: float = 10,
+    filepath: str = "",
 ) -> None:
     """
 
@@ -455,16 +485,9 @@ def upload_dataset(
 
     file_size = 0
     if schema.metadata.modality == Modality.image:
-        filepath_column = schema.metadata.filepath_column
-        assert filepath_column is not None
-        check_filepath_column(
-            modality=schema.metadata.modality,
-            dataset_filepath=filepath,
-            filepath_column=filepath_column,
-        )
-        file_size = get_image_dataset_size(
-            dataset_filepath=filepath, filepath_column=filepath_column
-        )
+        file_size = get_image_dataset_size(dataset=dataset, schema=schema)
+        if filepath:
+            get_file_size(filepath)
     else:
         file_size = get_file_size(filepath)
 
@@ -476,7 +499,7 @@ def upload_dataset(
     )
 
     # NOTE: makes simplifying assumption that first row size is representative of all row sizes
-    row_size = getsizeof(next(init_dataset_from_filepath(filepath).read_streaming_records()))
+    row_size = getsizeof(next(dataset.read_streaming_records()))
     rows_per_payload = int(payload_size * 1e6 / row_size)
     upload_queue: queue.Queue[Optional[List[Any]]] = queue.Queue(maxsize=2 * rows_per_payload)
 
@@ -484,7 +507,7 @@ def upload_dataset(
     validation_thread = threading.Thread(
         target=validate_rows,
         kwargs={
-            "dataset_filepath": filepath,
+            "dataset": dataset,
             "schema": schema,
             "log": log,
             "upload_queue": upload_queue,
@@ -492,6 +515,8 @@ def upload_dataset(
         },
     )
 
+    filepath_columns = get_dataset_filepath_columns(dataset, schema)
+
     # start and join processes
     validation_thread.start()
     asyncio.run(
@@ -502,6 +527,7 @@ def upload_dataset(
             schema=schema,
             upload_queue=upload_queue,
             rows_per_payload=rows_per_payload,
+            filepath_columns=filepath_columns,
         )
     )
     validation_thread.join()
@@ -610,7 +636,6 @@ def process_dataset(
     seen_ids: Set[str],
     existing_ids: Set[str],
     log: WarningLog,
-    dataset_dir: pathlib.Path,
     upload_queue: Optional["queue.Queue[Optional[List[Any]]]"] = None,
 ) -> None:
     """
@@ -620,9 +645,7 @@ def process_dataset(
     for record in tqdm(
         dataset.read_streaming_records(), total=len(dataset), initial=0, leave=True, unit=" rows"
     ):
-        row, row_id, warnings = validate_and_process_record(
-            record, schema, seen_ids, existing_ids, base_directory=dataset_dir
-        )
+        row, row_id, warnings = validate_and_process_record(record, schema, seen_ids, existing_ids)
         update_log_with_warnings(log, row_id, warnings)
         # row and row ID both present, i.e. row will be uploaded
         if row_id:
@@ -632,12 +655,17 @@ def process_dataset(
             upload_queue.put(list(row.values()), block=True)
 
 
-def get_image_dataset_size(dataset_filepath: str, filepath_column: str) -> int:
+def get_image_dataset_size(
+    dataset: Union[Dataset[IO[bytes]], Dataset[IO[str]]], schema: Schema
+) -> int:
     """Returns total image dataset size by summing file sizes of each image"""
-    dataset = init_dataset_from_filepath(dataset_filepath)
+    filepath_columns = get_dataset_filepath_columns(dataset, schema)
     return sum(
-        [
-            get_file_size(filepath=record[filepath_column], ignore_missing_files=True)
-            for record in dataset.read_streaming_records()
-        ]
+        sum(
+            [
+                get_file_size(filepath=record[filepath_column], ignore_missing_files=True)
+                for record in dataset.read_streaming_records()
+            ]
+        )
+        for filepath_column in filepath_columns
     )
